# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p

if __debug__:
    try:
        from typing import Dict, List  # noqa: F401
        from typing_extensions import Literal  # noqa: F401
    except ImportError:
        pass


class DebugLinkRecordScreen(p.MessageType):
    MESSAGE_WIRE_TYPE = 9003

    def __init__(
        self,
        target_directory: str = None,
    ) -> None:
        self.target_directory = target_directory

    @classmethod
    def get_fields(cls) -> Dict:
        return {
            1: ('target_directory', p.UnicodeType, 0),
        }
